# ==========================================================
# Author : Mikomi
# ==========================================================
# Version : 0.1
# Create : 2007/10/10 
# Last version : 2007/10/10
# ==========================================================
# Style class
# ==========================================================
# Description :
# A Style define the font, color background and foreground
# for an element, cell or label of the application
# ==========================================================

puts "require Style"

# ----------------------------------------------------------
require 'gtk2'
require $root+ "gui/GUI"
require $root+ "Common"
# ----------------------------------------------------------

puts "++ Style"


module GUI
  
  class Style
    include Common
    
    DEFAULT_DARK_VALUE = -4000
    DEFAULT_COLOR_FOREGROUND = "#000000"
    DEFAULT_COLOR_BACKGROUND = "#F0F0FF"
    DEFAULT_SELECTED_COLOR_BACKGROUND = "#FFE0E0"
    DEFAULT_FONT = "normal"
    
    attr_reader :name, :font, :foreground, :background, :selectedBackground, :darkerValue
    
    def initialize(pName, pFont=nil, pForeground=nil, pBackground=nil)
      @name = pName
      self.font = pFont
      self.foreground = pForeground
      self.background = pBackground
      self.selectedBackground = pBackground
      @darkerValue = DEFAULT_DARK_VALUE
    end
    
    # set the given font for this style, if nil, then set the default font
    def font=(pFont = nil)
      if pFont
        @font = pFont
      else
        @font = DEFAULT_FONT
      end      
    end

    # set the foreground color with the given color (#RRGGBB)
    def foreground=(pForeground)
      begin
        if pForeground
          @foreground = Gdk::Color.parse(pForeground)
        else
          @foreground = Gdk::Color.parse(DEFAULT_COLOR_FOREGROUND)
        end
      rescue StandardError => err
        @foreground = Gdk::Color.parse(DEFAULT_COLOR_FOREGROUND)
      end
    end

    # set the background color with the given color (#RRGGBB)
    def background=(pBackground)
      begin
        if pBackground
          @background = Gdk::Color.parse(pBackground)
        else
          @background = Gdk::Color.parse(DEFAULT_COLOR_BACKGROUND)
        end
      rescue StandardError => err
        @background = Gdk::Color.parse(DEFAULT_COLOR_BACKGROUND)
      end
    end
    
    # set the selected background color with the given color (#RRGGBB)
    def selectedBackground=(pBackground)
      begin
        if pBackground
          @selectedBackground = Gdk::Color.parse(pBackground)
        else
          @selectedBackground = Gdk::Color.parse(DEFAULT_SELECTED_COLOR_BACKGROUND)
        end
      rescue StandardError => err
        @selectedBackground = Gdk::Color.parse(DEFAULT_SELECTED_COLOR_BACKGROUND)
      end
    end

  end
end

puts "-- Style"

