
# ==========================================================
# Author : Mikomi
# ==========================================================
# Version : 0.1
# Create : 2007/10/23
# Last version : 2007/10/23
# ==========================================================
# Module Expression
# ==========================================================
# Description :
# Expression used for Rules evaluation
# ==========================================================

puts "require Expression"

# ----------------------------------------------------------
require $root+ "Common"
# ----------------------------------------------------------

puts "++ Expression"

# module experssion for karaoke
module Expression

  # Abstract Expression
  class Expression
    include Common
    attr_reader :name
    def initialize()
      @name = self.class.name.to_s
      @subExpressionList = Array.new
      set_debug_lvl(100)
    end
    def activeDebug()
      set_debug_lvl(25)
      @subExpressionList.each { |bSubExpression|
        bSubExpression.activeDebug()
      }
      return self
    end
    def setName(pName)
      @name = pName
      return self
    end
    def <<(pExpression)
      @subExpressionList << pExpression
    end
    def eval(pContext, pKaraoke = nil)
      v = doEval(pContext, pKaraoke)
      vStr = 'nil'
      vStr = v.to_s unless v.nil?
      log("eval "+self.to_s+' = '+vStr)
      return v
    end
    def doEval(pContext, pKaraoke)
      log('do nothing')
    end
    def to_s()
      subStr = ''
      subStr = '('+@subExpressionList.join(' ; ')+')' unless @subExpressionList.empty?
      return @name+subStr
    end
  end

  # ---------------------------------------
  # Context Expression
  # ---------------------------------------

  # Return a constant value
  class EConstant < Expression
    attr_accessor :value
    def initialize(pValue = nil)
      super()
      @value = pValue
    end
    def doEval(pContext, pKaraoke)
      return @value
    end
    def to_s()
      v = @value
      v = 'nil' if v.nil?
      return @name+'='+v.to_s
    end
  end

  # return the size of the current playlist
  class EPlaylistSize < Expression
    def doEval(pContext, pKaraoke)
      return pContext.playlist.size if pContext.playlist
      return 0
    end
  end

  # ---------------------------------------
  # Karaoke Expression
  # ---------------------------------------
  
  # return the value of the given column
  class EColumnValue < Expression
    attr_accessor :column
    def initialize(pColumn)
      super()
      @column = pColumn
    end
    def doEval(pContext, pKaraoke)
      return pKaraoke[@column]
    end
    def to_s
      return @name+'['+@column.to_s+']'
    end
  end

  # ---------------------------------------

  # sum each sub expression of this expression
  class EOpAdd < Expression
    def doEval(pContext, pKaraoke)
      total = 0
      @subExpressionList.each { |bSubExpression|
        v = bSubExpression.eval(pContext, pKaraoke)
        begin
          total += v
        rescue
          v = 'nil' if v.nil?
          log("can't add "+v.to_s+" to "+total.to_s)
        end
      }
      return total
    end
    def to_s()
      subStr = ''
      subStr = '( '+@subExpressionList.join(' + ')+' )' unless @subExpressionList.empty?
      return @name+subStr
    end
  end

end

puts "-- Expression"
