
# ==========================================================
# Author : Mikomi
# ==========================================================
# Version : 0.1
# Create : 2007/10/23
# Last version : 2007/10/23
# ==========================================================
# Update rules service class
# ==========================================================
# Description :
# Apply rules for the given context
# ==========================================================

puts "require UpdateRulesService"

# ----------------------------------------------------------
require $root+ "Common"
require $root+ "constants/CstsKaraoke"
require $root+ "helper/Rules"
require $root+ "helper/Expressions"
# ----------------------------------------------------------

puts "++ UpdateRulesService"

class UpdateRulesService
  include Common
  

  # ============================================================
	public
  # ============================================================
  
  def initialize()
    @resetRules = [
      rr_used(),
      rr_last(),
    ]
    
    @playlistRules = [
      pr_used(),
      pr_last(),
    ]

    @globalRules = [
      gr_last(),
    ]
    
    @contextRules = [
      cr_totalUsed()
    ]
  end

  # apply update rules defined for the given context
  # In order :
  #   - On each Karaoke :
  #       Apply Globale_Rules OR Playlist_Rules
  #   - Apply Context_Rules
  def applyUpdateRules(pContext)
    log("apply Update Rules") {
      pContext.globalList.each { |bKaraoke|
        applyUpdateKaraoke(pContext, bKaraoke)
      }
      @contextRules.each { |bContextRule|
        bContextRule.call(pContext)
      }
    }
  end


  # reset information link to this updateRules
  def resetUpdateRules(pContext)
    pContext.globalList.each { |bKaraoke|
      @resetRules.each { |bRule|
        bRule.call(pContext, bKaraoke)
      }
    }
  end

  # ============================================================
	private
  # ============================================================
  
  # apply rules on the given karaoke with the given context
  def applyUpdateKaraoke(pContext, pKaraoke)
    log("Treat : "+pKaraoke.to_s) {
      if pContext.playlist.contains(pKaraoke)
        # apply rules for playlist elements
        @playlistRules.each { |bRule|
          bRule.call(pContext, pKaraoke)
        }
      else
        # apply rules for other elements
        @globalRules.each { |bRule|
          bRule.call(pContext, pKaraoke)
        }
      end
    }
  end
  
  # ============================================================
  # Reset Rules
  # ============================================================
  
  # reset the parameter used
  # pKaraoke : Karaoke to reset
  # pContext : current ContextRules
  def rr_used()
    r = Rule::RResetColumn.new(CstsKaraoke::K_USED)
    r.setName("R_Used")
    return r
  end

  # reset the parameter used
  # pKaraoke : Karaoke to reset
  # pContext : current ContextRules
  def rr_last()
    r = Rule::RResetColumn.new(CstsKaraoke::K_LAST)
    r.setName("R_Last")
    return r
  end

  # ============================================================
  # Playlist Rules
  # ============================================================

  # update the parameter used
  def pr_used()
    r= Rule::RIncrementColumn.new(CstsKaraoke::K_USED)
    r.setName("P_Used")
    return r
  end
  
  # update the parameter last
  def pr_last()
    e = Expression::EConstant.new(0)
    r = Rule::RAffectColumn.new(CstsKaraoke::K_LAST, e)
    r.setName("P_Last")
    return r
  end

  # ============================================================
  # Globale Rules
  # ============================================================

  # update the parameter last
  def gr_last()
    #~ e = Expression::EOpAdd.new().setName('Add')
    #~ e << Expression::EColumnValue.new(CstsKaraoke::K_LAST).setName('column')
    #~ e << Expression::EPlaylistSize.new().setName('increment')
    
    #~ r = Rule::RAffectColumn.new(CstsKaraoke::K_LAST, e)
    #~ r.activeDebug()
    e = Expression::EPlaylistSize.new()
    r = Rule::RIncrementColumn.new(CstsKaraoke::K_LAST, e)
    r.setName("G_Last")
    return r
  end

  # ============================================================
  # Context Rules
  # ============================================================

  # update the total used title property of the context
  def cr_totalUsed()
    r = Rule::RUpdateTotalUsed.new()
    r.setName("C_TotalUsed")
    return r
  end

end

puts "-- UpdateRulesService"
